@@ -16,13 +16,13 @@ def checkfiles(path):
 			print e
 			return
 
-		entry.setLocale(None)
-		print entry.getName()
+		entry.setLocale("C")
+		entry.getName()
 
-		#try:
-		#	entry.validate()
-		#except ValidationError, e:
-		#	print e
+		try:
+			entry.validate()
+		except ValidationError, e:
+			print e
 
 try:
 	checkfiles(sys.argv[1])
