@@ -4,7 +4,7 @@ http://standards.freedesktop.org/menu-spec/
 """
 
 from __future__ import generators
-import os, xml.dom.minidom
+import locale, os, xml.dom.minidom
 
 from xdg.BaseDirectory import *
 from xdg.DesktopEntry import *
@@ -82,7 +82,7 @@ class Menu:
 
 	# FIXME: Performance: cache getName()
 	def __cmp__(self, other):
-		return cmp(self.getName(), other.getName())
+		return locale.strcoll(self.getName(), other.getName())
 
 	def __eq__(self, other):
 		if self.Name == str(other):
@@ -334,7 +334,7 @@ def do(menuentries, type, run):
 
 	def parseFilename(self, value):
 		self.parseNew()
-		self.Rule += "menuentry.DesktopFileID == '%s'" % value.strip()
+		self.Rule += "menuentry.DesktopFileID == '%s'" % value.strip().replace("\\", r"\\").replace("'", r"\'")
 
 	def parseCategory(self, value):
 		self.parseNew()
@@ -382,7 +382,7 @@ class MenuEntry:
 		self.DesktopEntry = DesktopEntry(os.path.join(dir,filename))
 		self.setAttributes(filename, dir, prefix)
 
-		# Can be one of Deleted/Hidden/Empty/NotShowIn or True
+		# Can be one of Deleted/Hidden/Empty/NotShowIn/NoExec or True
 		self.Show = True
 
 		# Semi-Private
@@ -443,7 +443,7 @@ class MenuEntry:
 		self.DesktopEntry.filename = os.path.join(dir, self.Filename)
 
 	def __cmp__(self, other):
-		return cmp(self.DesktopEntry.getName(), other.DesktopEntry.getName())
+		return locale.strcoll(self.DesktopEntry.getName(), other.DesktopEntry.getName())
 
 	def __eq__(self,other):
 		if self.DesktopFileID == str(other):
@@ -750,8 +750,11 @@ def __parseMergeDir(value, child, filename, parent):
 	value = __check(value, filename, "dir")
 	if value:
 		for item in os.listdir(value):
-			if os.path.splitext(item)[1] == ".menu":
-				__mergeFile(os.path.join(value, item), child, parent)
+			try:
+				if os.path.splitext(item)[1] == ".menu":
+					__mergeFile(os.path.join(value, item), child, parent)
+			except UnicodeDecodeError:
+				continue
 
 def __parseDefaultMergeDirs(child, filename, parent):
 	basename = os.path.splitext(os.path.basename(filename))[0]
@@ -800,10 +803,13 @@ def __mergeLegacyDir(dir, prefix, filename, parent):
 		m.NotInXml = True
 
 		for item in os.listdir(dir):
-			if item == ".directory":
-				m.Directories.append(item)
-			elif os.path.isdir(os.path.join(dir,item)):
-				m.addSubmenu(__mergeLegacyDir(os.path.join(dir,item), prefix, filename, parent))
+			try:
+				if item == ".directory":
+					m.Directories.append(item)
+				elif os.path.isdir(os.path.join(dir,item)):
+					m.addSubmenu(__mergeLegacyDir(os.path.join(dir,item), prefix, filename, parent))
+			except UnicodeDecodeError:
+				continue
 
 		tmp["cache"].addMenuEntries([dir],prefix, True)
 		menuentries = tmp["cache"].getMenuEntries([dir], False)
@@ -914,7 +920,7 @@ def sort(menu):
 					if submenu.Name not in tmp_s:
 						__parse_inline(submenu, menu)
 
-	# getHidden / NoDisplay / OnlyShowIn / NotOnlyShowIn / Deleted
+	# getHidden / NoDisplay / OnlyShowIn / NotOnlyShowIn / Deleted / NoExec
 	for entry in menu.Entries:
 		entry.Show = True
 		menu.Visible += 1
@@ -936,6 +942,9 @@ def sort(menu):
 			elif entry.DesktopEntry.getHidden() == True:
 				entry.Show = "Hidden"
 				menu.Visible -= 1
+			elif entry.DesktopEntry.getTryExec() and not __try_exec(entry.DesktopEntry.getTryExec()):
+				entry.Show = "NoExec"
+				menu.Visible -= 1
 			elif xdg.Config.windowmanager:
 				if ( entry.DesktopEntry.getOnlyShowIn() != [] and xdg.Config.windowmanager not in entry.DesktopEntry.getOnlyShowIn() ) \
 				or xdg.Config.windowmanager in entry.DesktopEntry.getNotShowIn():
@@ -960,6 +969,19 @@ def sort(menu):
 			if entry.NotInXml == True:
 				menu.Entries.remove(entry)
 
+def __try_exec(executable):
+	paths = os.environ['PATH'].split(os.pathsep)
+	if not os.path.isfile(executable):
+		for p in paths:
+			f = os.path.join(p, executable)
+			if os.path.isfile(f):
+				if os.access(f, os.X_OK):
+					return True
+	else:
+		if os.access(executable, os.X_OK):
+			return True
+	return False
+
 # inline tags
 def __parse_inline(submenu, menu):
 	if submenu.Layout.inline == "true":
@@ -1022,14 +1044,17 @@ class MenuEntryCache:
 			pass
 		for dir in appdirs:
 			for menuentry in self.cacheEntries[dir]:
-				if menuentry.DesktopFileID not in ids:
-					ids.append(menuentry.DesktopFileID)
-					list.append(menuentry)
-				elif menuentry.getType() == "System":
-				# FIXME: This is only 99% correct, but still...
-					i = list.index(menuentry)
-					e = list[i]
-					if e.getType() == "User":
-						e.Original = menuentry
+				try:
+					if menuentry.DesktopFileID not in ids:
+						ids.append(menuentry.DesktopFileID)
+						list.append(menuentry)
+					elif menuentry.getType() == "System":
+					# FIXME: This is only 99% correct, but still...
+						i = list.index(menuentry)
+						e = list[i]
+						if e.getType() == "User":
+							e.Original = menuentry
+				except UnicodeDecodeError:
+					continue
 		self.cache[key] = list
 		return list
